@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 3, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #include "mainwindow.h"
 #include "transferitemdelegate.h"
 #include "addurlsdialog.h"
@@ -8,6 +25,7 @@
 #include "transferpropertiesdialog.h"
 #include "packagepropertiesdialog.h"
 #include "../shared/database.h"
+#include "../shared/urlchecker.h"
 #include "../shared/urlretriever.h"
 #include "../shared/pluginmanager.h"
 #include "../shared/transfermodel.h"
@@ -301,23 +319,22 @@ MainWindow::MainWindow(QWidget *parent) :
     this->connect(m_packageMenu, SIGNAL(aboutToShow()), this, SLOT(setPackageMenuActions()));
     this->connect(m_optionsButton, SIGNAL(clicked()), this, SLOT(showOptionsMenu()));
     this->connect(m_model, SIGNAL(countChanged(int)), this, SLOT(onPackageCountChanged(int)));
+    this->connect(m_model, SIGNAL(nextActionChanged(Transfers::Action)), this, SLOT(onNextActionChanged(Transfers::Action)));
     this->connect(m_trayAddUrlsAction, SIGNAL(triggered()), this, SLOT(showAddUrlsDialog()));
     this->connect(m_trayImportUrlsAction, SIGNAL(triggered()), this, SLOT(showTextFileDialog()));
     this->connect(m_trayRetrieveUrlsAction, SIGNAL(triggered()), this, SLOT(showRetrieveUrlsDialog()));
     this->connect(m_trayPreferencesAction, SIGNAL(triggered()), this, SLOT(showSettingsDialog()));
     this->connect(m_trayQuitAction, SIGNAL(triggered()), QCoreApplication::instance(), SLOT(quit()));
     this->connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(onTrayIconActivated(QSystemTrayIcon::ActivationReason)));
+    this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)), this, SLOT(onMaximumConcurrentTransfersChanged(int, int)));
+    this->connect(Settings::instance(), SIGNAL(maximumConnectionsPerTransferChanged(int,int)), this, SLOT(onGlobalTransferConnectionsChanged(int, int)));
+    this->connect(Settings::instance(), SIGNAL(downloadRateLimitChanged(int)), this, SLOT(onDownloadRateLimitChanged(int)));
     this->connect(Database::instance(), SIGNAL(categoriesChanged()), this, SLOT(setCategoryMenuActions()));
     this->connect(ClipboardMonitor::instance(), SIGNAL(clipboardUrlsReady(QString)), this, SLOT(showAddUrlsDialog(QString)));
     this->connect(PluginManager::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
     this->connect(PluginManager::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
     this->connect(PluginManager::instance(), SIGNAL(pluginsReady()), this, SLOT(onPluginsReady()));
     this->connect(PluginManager::instance(), SIGNAL(pluginsReady()), this, SLOT(hideProgressDialog()));
-    this->connect(UrlRetriever::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
-    this->connect(UrlRetriever::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
-    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(onUrlRetrieverFinished()));
-    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(hideProgressDialog()));
-    this->connect(UrlRetriever::instance(), SIGNAL(cancelled()), this, SLOT(hideProgressDialog()));
 
     this->onWindowStateChanged();
     this->onPackageCountChanged(m_model->rowCount());
@@ -404,6 +421,10 @@ void MainWindow::onNextActionIndexChanged(int index) {
     m_model->setNextAction(static_cast<Transfers::Action>(m_nextActionComboBox->itemData(index, Qt::UserRole + 1).toInt()));
 }
 
+void MainWindow::onNextActionChanged(Transfers::Action action) {
+    m_nextActionComboBox->setCurrentIndex(m_nextActionComboBox->findData(action, Qt::UserRole + 1));
+}
+
 void MainWindow::onPackageCountChanged(int count) {
     m_transferMenu->setEnabled(count > 0);
     m_packageMenu->setEnabled(count > 0);
@@ -424,18 +445,53 @@ void MainWindow::setMaximumConcurrentTransfers() {
     }
 }
 
+void MainWindow::onMaximumConcurrentTransfersChanged(int oldMax, int newMax) {
+    if (oldMax == newMax) {
+        return;
+    }
+
+    foreach (QAction *action, m_concurrentGroup->actions()) {
+        if (action->data().toInt() == newMax) {
+            action->setChecked(true);
+            return;
+        }
+    }
+}
+
 void MainWindow::setGlobalTransferConnections() {
     if (QAction *action = m_connectionsGroup->checkedAction()) {
         Settings::instance()->setMaximumConnectionsPerTransfer(action->data().toInt());
     }
 }
 
+void MainWindow::onGlobalTransferConnectionsChanged(int oldMax, int newMax) {
+    if (oldMax == newMax) {
+        return;
+    }
+
+    foreach (QAction *action, m_connectionsGroup->actions()) {
+        if (action->data().toInt() == newMax) {
+            action->setChecked(true);
+            return;
+        }
+    }
+}
+
 void MainWindow::setDownloadRateLimit() {
     if (QAction *action = m_rateLimitGroup->checkedAction()) {
         Settings::instance()->setDownloadRateLimit(action->data().toInt());
     }
 }
 
+void MainWindow::onDownloadRateLimitChanged(int limit) {
+    foreach (QAction *action, m_rateLimitGroup->actions()) {
+        if (action->data().toInt() == limit) {
+            action->setChecked(true);
+            return;
+        }
+    }
+}
+
 void MainWindow::updateSpeed(int speed) {
     m_speedLabel->setText(QString::number(speed) + " kB/s");
 }
@@ -721,6 +777,12 @@ void MainWindow::showAddUrlsDialog(const QString &text, const QString &fileName)
     }
 
     dialog->open();
+    this->connect(dialog, SIGNAL(urlsAvailable(QString)), this, SLOT(addUrlsFromText(QString)));
+}
+
+void MainWindow::addUrlsFromText(const QString &text) {
+    m_checkDialog->open();
+    UrlChecker::instance()->parseUrlsFromText(text);
 }
 
 void MainWindow::showRetrieveUrlsDialog(const QString &text, const QString &fileName) {
@@ -734,6 +796,17 @@ void MainWindow::showRetrieveUrlsDialog(const QString &text, const QString &file
     }
 
     dialog->open();
+    this->connect(dialog, SIGNAL(urlsAvailable(QString)), this, SLOT(retrieveUrlsFromText(QString)));
+}
+
+void MainWindow::retrieveUrlsFromText(const QString &text) {
+    this->disconnect(UrlRetriever::instance(), 0, this, 0);
+    this->connect(UrlRetriever::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
+    this->connect(UrlRetriever::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
+    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(onUrlRetrieverFinished()));
+    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(hideProgressDialog()));
+    this->connect(UrlRetriever::instance(), SIGNAL(cancelled()), this, SLOT(hideProgressDialog()));
+    UrlRetriever::instance()->parseUrlsFromText(text);
 }
 
 void MainWindow::onUrlRetrieverFinished() {
@@ -746,6 +819,8 @@ void MainWindow::onUrlRetrieverFinished() {
         this->showAddUrlsDialog(results);
         UrlRetriever::instance()->clearResults();
     }
+
+    this->disconnect(UrlRetriever::instance(), 0, this, 0);
 }
 
 void MainWindow::showTextFileDialog() {
