@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 3, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #include "mainwindow.h"
 #include "transferitemdelegate.h"
 #include "valueselectoraction.h"
@@ -153,16 +170,14 @@ MainWindow::MainWindow(QWidget *parent) :
     this->connect(m_model, SIGNAL(totalDownloadSpeedChanged(int)), this, SLOT(updateSpeed(int)));
     this->connect(m_model, SIGNAL(countChanged(int)), this, SLOT(onPackageCountChanged(int)));
     this->connect(m_contextMenu, SIGNAL(aboutToShow()), this, SLOT(setTransferMenuActions()));
+    this->connect(Settings::instance(), SIGNAL(maximumConcurrentTransfersChanged(int,int)), this, SLOT(onMaximumConcurrentTransfersChanged(int, int)));
+    this->connect(Settings::instance(), SIGNAL(maximumConnectionsPerTransferChanged(int,int)), this, SLOT(onGlobalTransferConnectionsChanged(int, int)));
+    this->connect(Settings::instance(), SIGNAL(downloadRateLimitChanged(int)), this, SLOT(onDownloadRateLimitChanged(int)));
     this->connect(ClipboardMonitor::instance(), SIGNAL(clipboardUrlsReady(QString)), this, SLOT(showAddUrlsDialog(QString)));
     this->connect(PluginManager::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
     this->connect(PluginManager::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
     this->connect(PluginManager::instance(), SIGNAL(pluginsReady()), this, SLOT(onPluginsReady()));
     this->connect(PluginManager::instance(), SIGNAL(pluginsReady()), this, SLOT(hideProgressDialog()));
-    this->connect(UrlRetriever::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
-    this->connect(UrlRetriever::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
-    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(onUrlRetrieverFinished()));
-    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(hideProgressDialog()));
-    this->connect(UrlRetriever::instance(), SIGNAL(cancelled()), this, SLOT(hideProgressDialog()));
 
     this->onPackageCountChanged(m_model->rowCount());
 }
@@ -186,6 +201,10 @@ void MainWindow::setNextAction(const QVariant &value) {
     m_model->setNextAction(static_cast<Transfers::Action>(value.toInt()));
 }
 
+void MainWindow::onNextActionChanged(Transfers::Action action) {
+    m_nextAction->setValue(action);
+}
+
 void MainWindow::setTransferFilter(const QVariant &value) {
     m_model->setStatusFilter(static_cast<Transfers::Status>(value.toInt()));
 }
@@ -194,14 +213,34 @@ void MainWindow::setMaximumConcurrentTransfers(const QVariant &value) {
     Settings::instance()->setMaximumConcurrentTransfers(value.toInt());
 }
 
+void MainWindow::onMaximumConcurrentTransfersChanged(int oldMax, int newMax) {
+    if (oldMax == newMax) {
+        return;
+    }
+
+    m_concurrentTransfersAction->setValue(newMax);
+}
+
 void MainWindow::setGlobalTransferConnections(const QVariant &value) {
     Settings::instance()->setMaximumConnectionsPerTransfer(value.toInt());
 }
 
+void MainWindow::onGlobalTransferConnectionsChanged(int oldMax, int newMax) {
+    if (oldMax == newMax) {
+        return;
+    }
+
+    m_connectionsAction->setValue(newMax);
+}
+
 void MainWindow::setDownloadRateLimit(const QVariant &value) {
     Settings::instance()->setDownloadRateLimit(value.toInt());
 }
 
+void MainWindow::onDownloadRateLimitChanged(int limit) {
+    m_rateLimitAction->setValue(limit);
+}
+
 void MainWindow::updateSpeed(int speed) {
     m_speedLabel->setText(QString::number(speed) + " kB/s");
 }
@@ -359,18 +398,12 @@ void MainWindow::showAddUrlsDialog(const QString &text, const QString &fileName)
     }
 
     dialog->open();
+    this->connect(dialog, SIGNAL(urlsAvailable(QString)), this, SLOT(addUrlsFromText(QString)));
 }
 
-void MainWindow::onUrlRetrieverFinished() {
-    QString results = UrlRetriever::instance()->resultsString();
-
-    if (results.isEmpty()) {
-        QMaemo5InformationBox::information(this, tr("No supported URLs found"));
-    }
-    else {
-        this->showAddUrlsDialog(results);
-        UrlRetriever::instance()->clearResults();
-    }
+void MainWindow::addUrlsFromText(const QString &text) {
+    m_checkDialog->open();
+    UrlChecker::instance()->parseUrlsFromText(text);
 }
 
 void MainWindow::showRetrieveUrlsDialog(const QString &text, const QString &fileName) {
@@ -384,6 +417,31 @@ void MainWindow::showRetrieveUrlsDialog(const QString &text, const QString &file
     }
 
     dialog->open();
+    this->connect(dialog, SIGNAL(urlsAvailable(QString)), this, SLOT(retrieveUrlsFromText(QString)));
+}
+
+void MainWindow::retrieveUrlsFromText(const QString &text) {
+    this->disconnect(UrlRetriever::instance(), 0, this, 0);
+    this->connect(UrlRetriever::instance(), SIGNAL(busy(QString,int)), this, SLOT(showProgressDialog(QString,int)));
+    this->connect(UrlRetriever::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
+    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(onUrlRetrieverFinished()));
+    this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(hideProgressDialog()));
+    this->connect(UrlRetriever::instance(), SIGNAL(cancelled()), this, SLOT(hideProgressDialog()));
+    UrlRetriever::instance()->parseUrlsFromText(text);
+}
+
+void MainWindow::onUrlRetrieverFinished() {
+    QString results = UrlRetriever::instance()->resultsString();
+
+    if (results.isEmpty()) {
+        QMaemo5InformationBox::information(this, tr("No supported URLs found"));
+    }
+    else {
+        this->showAddUrlsDialog(results);
+        UrlRetriever::instance()->clearResults();
+    }
+
+    this->disconnect(UrlRetriever::instance(), 0, this, 0);
 }
 
 void MainWindow::showTextFileDialog() {
