@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 3, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #ifndef SETTINGS_H
 #define SETTINGS_H
 
@@ -82,6 +99,20 @@ class Settings : public QSettings
                READ networkProxyPassword
                WRITE setNetworkProxyPassword
                NOTIFY networkProxyPasswordChanged)
+#ifdef WEB_INTERFACE
+    Q_PROPERTY(bool enableWebInterface
+               READ enableWebInterface
+               WRITE setEnableWebInterface
+               NOTIFY enableWebInterfaceChanged)
+    Q_PROPERTY(quint16 webInterfacePort
+               READ webInterfacePort
+               WRITE setWebInterfacePort
+               NOTIFY webInterfacePortChanged)
+    Q_PROPERTY(QString webInterfaceTheme
+               READ webInterfaceTheme
+               WRITE setWebInterfaceTheme
+               NOTIFY webInterfaceThemeChanged)
+#endif
 
 public:
     inline bool startTransfersAutomatically() const { return m_automatic; }
@@ -102,6 +133,11 @@ public:
     inline quint16 networkProxyPort() const { return m_proxyPort; }
     inline QString networkProxyUser() const { return m_proxyUser; }
     inline QString networkProxyPassword() const { return m_proxyPass; }
+#ifdef WEB_INTERFACE
+    inline bool enableWebInterface() const { return m_enableWebIf; }
+    inline quint16 webInterfacePort() const { return m_webIfPort; }
+    inline QString webInterfaceTheme() const { return m_webIfTheme; }
+#endif
     Q_INVOKABLE QVariant setting(const QString &key) const;
 
     static Settings* instance();
@@ -129,6 +165,11 @@ public slots:
     void setNetworkProxyUser(const QString &user);
     void setNetworkProxyPassword(const QString &password);
     void setNetworkProxy();
+#ifdef WEB_INTERFACE
+    void setEnableWebInterface(bool enable);
+    void setWebInterfacePort(quint16 port);
+    void setWebInterfaceTheme(const QString &theme);
+#endif
     void setSetting(const QString &key, const QVariant &value);
 
 private:
@@ -157,9 +198,14 @@ signals:
     void networkProxyPortChanged(quint16 port);
     void networkProxyUserChanged(const QString &user);
     void networkProxyPasswordChanged(const QString &password);
+#ifdef WEB_INTERFACE
+    void enableWebInterfaceChanged(bool enable);
+    void webInterfacePortChanged(quint16 port);
+    void webInterfaceThemeChanged(const QString &theme);
+#endif
 
 private:
-    static Settings *m_instance;
+    static Settings *self;
 
     bool m_automatic;
     QString m_path;
@@ -180,6 +226,11 @@ private:
     quint16 m_proxyPort;
     QString m_proxyUser;
     QString m_proxyPass;
+#ifdef WEB_INTERFACE
+    bool m_enableWebIf;
+    quint16 m_webIfPort;
+    QString m_webIfTheme;
+#endif
 };
 
 #endif // SETTINGS_H
