@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2014 Stuart Howarth <showarth@marxoft.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU Lesser General Public License,
+ * version 3, as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #include "keeptoshare.h"
 #include "json.h"
 #include <QNetworkAccessManager>
@@ -19,7 +36,7 @@ KeepToShare::KeepToShare(QObject *parent) :
 }
 
 QRegExp KeepToShare::urlPattern() const {
-    return QRegExp("http(s|)://(www.|)(keep2share|k2s).cc/file/\\w+", Qt::CaseInsensitive);
+    return QRegExp("http(s|)://(www.|)(keep2s(hare|)|k2s).cc/file/\\w+", Qt::CaseInsensitive);
 }
 
 bool KeepToShare::urlSupported(const QUrl &url) const {
@@ -52,7 +69,7 @@ void KeepToShare::checkKeep2ShareLogin() {
     switch (statusCode) {
     case 200:
     case 201:
-        this->loginK2C(m_user, m_pass);
+        this->loginK2S(m_user, m_pass);
         break;
     default:
         m_connections = 1;
@@ -60,24 +77,56 @@ void KeepToShare::checkKeep2ShareLogin() {
         break;
     }
 
-    m_user = "";
-    m_pass = "";
-
     reply->deleteLater();
 }
 
-void KeepToShare::loginK2C(const QString &username, const QString &password) {
+void KeepToShare::loginK2S(const QString &username, const QString &password) {
     QString data = QString("LoginForm[username]=%1&LoginForm[password]=%2&LoginForm[rememberMe]=1").arg(username).arg(password);
     QUrl url("http://k2s.cc/login.html");
     QNetworkRequest request(url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
     request.setRawHeader("X-Requested-With", "XMLHttpRequest");
     QNetworkReply *reply = this->networkAccessManager()->post(request, data.toUtf8());
-    this->connect(reply, SIGNAL(finished()), this, SLOT(checkK2CLogin()));
+    this->connect(reply, SIGNAL(finished()), this, SLOT(checkK2SLogin()));
     this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
-void KeepToShare::checkK2CLogin() {
+void KeepToShare::checkK2SLogin() {
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
+
+    if (!reply) {
+        emit error(NetworkError);
+        return;
+    }
+
+    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+    switch (statusCode) {
+    case 200:
+    case 201:
+        this->loginKeep2s(m_user, m_pass);
+        break;
+    default:
+        m_connections = 1;
+        emit loggedIn(false);
+        break;
+    }
+
+    reply->deleteLater();
+}
+
+void KeepToShare::loginKeep2s(const QString &username, const QString &password) {
+    QString data = QString("LoginForm[username]=%1&LoginForm[password]=%2&LoginForm[rememberMe]=1").arg(username).arg(password);
+    QUrl url("http://keep2s.cc/login.html");
+    QNetworkRequest request(url);
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+    request.setRawHeader("X-Requested-With", "XMLHttpRequest");
+    QNetworkReply *reply = this->networkAccessManager()->post(request, data.toUtf8());
+    this->connect(reply, SIGNAL(finished()), this, SLOT(checkKeep2sLogin()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
+}
+
+void KeepToShare::checkKeep2sLogin() {
     QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
@@ -99,12 +148,14 @@ void KeepToShare::checkK2CLogin() {
         break;
     }
 
+    m_user = "";
+    m_pass = "";
+
     reply->deleteLater();
 }
 
-void KeepToShare::checkUrl(const QUrl &webUrl) {
-    QNetworkRequest request(webUrl);
-    request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
+void KeepToShare::checkUrl(const QUrl &url) {
+    QNetworkRequest request(url);
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkUrlIsValid()));
     this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
@@ -119,7 +170,7 @@ void KeepToShare::checkUrlIsValid() {
     }
 
     QString redirect = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
-    QRegExp re("http(s|)://keep2share.cc/file/url.html[^'\"]+");
+    QRegExp re("(http://(keep2s(hare|)|k2s).cc)/file/url.html[^'\"]+");
 
     if ((!redirect.isEmpty()) && (re.indexIn(redirect) == -1)) {
         this->checkUrl(QUrl(redirect));
@@ -146,10 +197,9 @@ void KeepToShare::checkUrlIsValid() {
     reply->deleteLater();
 }
 
-void KeepToShare::getDownloadRequest(const QUrl &webUrl) {
+void KeepToShare::getDownloadRequest(const QUrl &url) {
     emit statusChanged(Connecting);
-    QNetworkRequest request(webUrl);
-    request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
+    QNetworkRequest request(url);
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(onWebPageDownloaded()));
     this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
@@ -163,36 +213,56 @@ void KeepToShare::onWebPageDownloaded() {
         return;
     }
 
-    QRegExp re("http(s|)://keep2share.cc/file/url.html[^'\"]+");
+    QRegExp re("(http://(keep2s(hare|)|k2s).cc)/file/url.html[^'\"]+");
     QString redirect = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
 
     if (re.indexIn(redirect) == 0) {
-        QNetworkRequest request;
-        request.setUrl(QUrl(re.cap()));
-        emit downloadRequestReady(request);
+        QUrl url(re.cap());
+
+        if (url.host().isEmpty()) {
+            url.setScheme("http");
+            url.setHost("keep2share.cc");
+        }
+
+        emit downloadRequestReady(QNetworkRequest(url));
     }
     else if (!redirect.isEmpty()) {
-        this->getDownloadRequest(QUrl(redirect));
+        QUrl url(redirect);
+
+        if (url.host().isEmpty()) {
+            url.setScheme("http");
+            url.setHost("keep2share.cc");
+        }
+
+        this->getDownloadRequest(url);
     }
     else {
         QString response(reply->readAll());
 
-        if (response.contains("/file/url.html?file=")) {
-	    QUrl url("http://keep2share.cc/file/url.html");
-            url.addQueryItem("file", response.section("/file/url.html?file=", 1, 1).section('\'', 0, 0));
+        if (re.indexIn(response) >= 0) {
+            QUrl url(re.cap());
+
+            if (url.host().isEmpty()) {
+                url.setScheme("http");
+                url.setHost("keep2share.cc");
+            }
+
             emit downloadRequestReady(QNetworkRequest(url));
         }
         else {
-            m_url = reply->request().url();
+            m_url = reply->url();
             m_fileId = response.section("slow_id\" value=\"", 1, 1).section('"', 0, 0);
 
             if (m_fileId.isEmpty()) {
-		if (response.contains("This file is available<br>only for premium members")) {
-		    emit error(Unauthorised);
-		}
-		else {
+                if (response.contains("only for premium members")) {
+                    emit error(Unauthorised);
+                }
+                else if (response.contains("not found or deleted")) {
+                    emit error(NotFound);
+                }
+                else {
                     emit error(UnknownError);
-		}
+                }
             }
             else {
                 this->getCaptchaKey();
@@ -206,7 +276,6 @@ void KeepToShare::onWebPageDownloaded() {
 void KeepToShare::getCaptchaKey() {
     QNetworkRequest request(m_url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-    request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
     request.setRawHeader("Referer", m_url.toString().toUtf8());
     QNetworkReply *reply = this->networkAccessManager()->post(request, "yt0=&slow_id=" + m_fileId.toUtf8());
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkCaptchaKey()));
@@ -222,14 +291,21 @@ void KeepToShare::checkCaptchaKey() {
     }
     
     QString response(reply->readAll());
-    QRegExp re("/file/url.html[^'\"]+");
+    QRegExp re("(http://(keep2s(hare|)|k2s).cc)/file/url.html[^'\"]+");
 
     if (re.indexIn(response) >= 0) {
-        QUrl url("http://keep2share.cc" + re.cap());
+        QUrl url(re.cap());
+
+        if (url.host().isEmpty()) {
+            url.setScheme("http");
+            url.setHost("keep2share.cc");
+        }
+
         emit downloadRequestReady(QNetworkRequest(url));
     }
     else {
-        m_captchaKey = response.section("http://www.google.com/recaptcha/api/challenge?k=", 1, 1).section('"', 0, 0);
+        m_url = reply->url();
+        m_captchaKey = response.section("/file/captcha.html?v=", 1, 1).section('"', 0, 0);
         
         if (m_captchaKey.isEmpty()) {
             QString waitString = response.section("Please wait", 1, 1).section("to download this file", 0, 0).trimmed();
@@ -258,11 +334,12 @@ void KeepToShare::checkCaptchaKey() {
 }
 
 void KeepToShare::submitCaptchaResponse(const QString &challenge, const QString &response) {
-    QString data = QString("CaptchaForm[code]=&recaptcha_challenge_field=%1&recaptcha_response_field=%2&free=1&freeDownloadRequest=1&uniqueId=%3&yt0=").arg(challenge).arg(response).arg(m_fileId);
+    Q_UNUSED(challenge);
 
+    QString data = QString("CaptchaForm[code]=%1&free=1&freeDownloadRequest=1&uniqueId=%2&yt0=").arg(response).arg(m_fileId);
     QNetworkRequest request(m_url);
-    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-    request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded; charset=UTF-8");
+    request.setRawHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
     request.setRawHeader("Referer", m_url.toString().toUtf8());
     QNetworkReply *reply = this->networkAccessManager()->post(request, data.toUtf8());
     this->connect(reply, SIGNAL(finished()), this, SLOT(onCaptchaSubmitted()));
@@ -285,7 +362,14 @@ void KeepToShare::onCaptchaSubmitted() {
         this->connect(this, SIGNAL(waitFinished()), this, SLOT(getDownloadRequest()));
     }
     else if (response.contains("The verification code is incorrect")) {
-        emit error(CaptchaError);
+        m_captchaKey = response.section("/file/captcha.html?v=", 1, 1).section('"', 0, 0);
+
+        if (m_captchaKey.isEmpty()) {
+            emit error(UnknownError);
+        }
+        else {
+            emit error(CaptchaError);
+        }
     }
     else {
         emit error(UnknownError);
@@ -295,13 +379,11 @@ void KeepToShare::onCaptchaSubmitted() {
 }
 
 void KeepToShare::getDownloadRequest() {
-    QString data = QString("uniqueId=%1&free=1").arg(m_fileId);
     QNetworkRequest request(m_url);
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded; charset=UTF-8");
-    request.setRawHeader("Accept-Language", "en-GB,en-US;q=0.8,en;q=0.6");
+    request.setRawHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
     request.setRawHeader("Referer", m_url.toString().toUtf8());
-    request.setRawHeader("X-Requested-With", "XMLHttpRequest");
-    QNetworkReply *reply = this->networkAccessManager()->post(request, data.toUtf8());
+    QNetworkReply *reply = this->networkAccessManager()->post(request, "free=1&uniqueId=" + m_fileId.toUtf8());
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkDownloadRequest()));
     this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
     this->disconnect(this, SIGNAL(waitFinished()), this, SLOT(getDownloadRequest()));
@@ -315,21 +397,41 @@ void KeepToShare::checkDownloadRequest() {
         return;
     }
 
-    QString response(reply->readAll());
+    QRegExp re("(http://(keep2s(hare|)|k2s).cc)/file/url.html[^'\"]+");
+    QString redirect = reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toString();
 
-    QVariantMap result = Json::parse(response).toMap();
-    QUrl url = result.value("url").toUrl();
+    if (re.indexIn(redirect) == 0) {
+        QUrl url(re.cap());
+
+        if (url.host().isEmpty()) {
+            url.setScheme("http");
+            url.setHost("keep2share.cc");
+        }
 
-    if (!url.isEmpty()) {
-	emit downloadRequestReady(QNetworkRequest(url));
-    }
-    else if (response.contains("/file/url.html?file=")) {
-        QUrl url("http://keep2share.cc/file/url.html");
-        url.addQueryItem("file", m_fileId);
         emit downloadRequestReady(QNetworkRequest(url));
     }
     else {
-        emit error(UnknownError);
+        QString response(reply->readAll());
+
+        QVariantMap result = Json::parse(response).toMap();
+        QUrl url = result.value("url").toUrl();
+
+        if (!url.isEmpty()) {
+            emit downloadRequestReady(QNetworkRequest(url));
+        }
+        else if (re.indexIn(response) >= 0) {
+            QUrl url(re.cap());
+
+            if (url.host().isEmpty()) {
+                url.setScheme("http");
+                url.setHost("keep2share.cc");
+            }
+
+            emit downloadRequestReady(QNetworkRequest(url));
+        }
+        else {
+            emit error(UnknownError);
+        }
     }
     
     reply->deleteLater();
